package sun.security.x509;

import sun.security.provider.X509Factory;
import sun.security.util.DerInputStream;
import java.util.Enumeration;
import sun.security.util.ObjectIdentifier;
import java.util.TreeSet;
import java.util.Set;
import java.math.BigInteger;
import java.security.Principal;
import java.security.cert.X509Certificate;
import java.security.cert.Certificate;
import sun.misc.HexDumpEncoder;
import java.security.PrivateKey;
import java.security.Provider;
import java.security.InvalidAlgorithmParameterException;
import java.security.ProviderException;
import sun.security.util.SignatureUtil;
import java.security.Signature;
import java.security.SignatureException;
import java.security.NoSuchProviderException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.Iterator;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.security.cert.CRLException;
import sun.security.util.DerValue;
import java.util.LinkedList;
import java.util.TreeMap;
import java.security.PublicKey;
import java.util.List;
import java.security.cert.X509CRLEntry;
import java.util.Map;
import java.util.Date;
import javax.security.auth.x500.X500Principal;
import sun.security.util.DerEncoder;
import java.security.cert.X509CRL;

public class X509CRLImpl extends X509CRL implements DerEncoder
{
    private byte[] signedCRL;
    private byte[] signature;
    private byte[] tbsCertList;
    private AlgorithmId sigAlgId;
    private int version;
    private AlgorithmId infoSigAlgId;
    private X500Name issuer;
    private X500Principal issuerPrincipal;
    private Date thisUpdate;
    private Date nextUpdate;
    private Map<X509CRLImpl.X509IssuerSerial, X509CRLEntry> revokedMap;
    private List<X509CRLEntry> revokedList;
    private CRLExtensions extensions;
    private static final boolean isExplicit = true;
    private boolean readOnly;
    private PublicKey verifiedPublicKey;
    private String verifiedProvider;
    
    private X509CRLImpl() {
        this.signedCRL = null;
        this.signature = null;
        this.tbsCertList = null;
        this.sigAlgId = null;
        this.issuer = null;
        this.issuerPrincipal = null;
        this.thisUpdate = null;
        this.nextUpdate = null;
        this.revokedMap = new TreeMap();
        this.revokedList = new LinkedList();
        this.extensions = null;
        this.readOnly = false;
    }
    
    public X509CRLImpl(byte[] encoding) throws CRLException {
        this.signedCRL = null;
        this.signature = null;
        this.tbsCertList = null;
        this.sigAlgId = null;
        this.issuer = null;
        this.issuerPrincipal = null;
        this.thisUpdate = null;
        this.nextUpdate = null;
        this.revokedMap = new TreeMap();
        this.revokedList = new LinkedList();
        this.extensions = null;
        this.readOnly = false;
        try {
            this.parse(new DerValue(encoding));
        }
        catch (IOException ex) {
            this.signedCRL = null;
            throw new CRLException("Parsing error: " + ex.getMessage());
        }
    }
    
    public X509CRLImpl(DerValue derValue) throws CRLException {
        this.signedCRL = null;
        this.signature = null;
        this.tbsCertList = null;
        this.sigAlgId = null;
        this.issuer = null;
        this.issuerPrincipal = null;
        this.thisUpdate = null;
        this.nextUpdate = null;
        this.revokedMap = new TreeMap();
        this.revokedList = new LinkedList();
        this.extensions = null;
        this.readOnly = false;
        try {
            this.parse(derValue);
        }
        catch (IOException ex) {
            this.signedCRL = null;
            throw new CRLException("Parsing error: " + ex.getMessage());
        }
    }
    
    public X509CRLImpl(InputStream in) throws CRLException {
        this.signedCRL = null;
        this.signature = null;
        this.tbsCertList = null;
        this.sigAlgId = null;
        this.issuer = null;
        this.issuerPrincipal = null;
        this.thisUpdate = null;
        this.nextUpdate = null;
        this.revokedMap = new TreeMap();
        this.revokedList = new LinkedList();
        this.extensions = null;
        this.readOnly = false;
        try {
            this.parse(new DerValue(in));
        }
        catch (IOException ex) {
            this.signedCRL = null;
            throw new CRLException("Parsing error: " + ex.getMessage());
        }
    }
    
    public X509CRLImpl(X500Name issuer, Date thisUpdate, Date nextUpdate) {
        this.signedCRL = null;
        this.signature = null;
        this.tbsCertList = null;
        this.sigAlgId = null;
        this.issuer = null;
        this.issuerPrincipal = null;
        this.thisUpdate = null;
        this.nextUpdate = null;
        this.revokedMap = new TreeMap();
        this.revokedList = new LinkedList();
        this.extensions = null;
        this.readOnly = false;
        this.issuer = issuer;
        this.thisUpdate = thisUpdate;
        this.nextUpdate = nextUpdate;
    }
    
    public X509CRLImpl(X500Name issuer, Date thisUpdate, Date nextUpdate, X509CRLEntry[] array) throws CRLException {
        this.signedCRL = null;
        this.signature = null;
        this.tbsCertList = null;
        this.sigAlgId = null;
        this.issuer = null;
        this.issuerPrincipal = null;
        this.thisUpdate = null;
        this.nextUpdate = null;
        this.revokedMap = new TreeMap();
        this.revokedList = new LinkedList();
        this.extensions = null;
        this.readOnly = false;
        this.issuer = issuer;
        this.thisUpdate = thisUpdate;
        this.nextUpdate = nextUpdate;
        if (array != null) {
            X500Principal x500Principal = this.getIssuerX500Principal();
            for (int i = 0; i < array.length; ++i) {
                X509CRLEntryImpl x509CRLEntryImpl = (X509CRLEntryImpl)array[i];
                try {
                    x500Principal = this.getCertIssuer(x509CRLEntryImpl, x500Principal);
                }
                catch (IOException cause) {
                    throw new CRLException(cause);
                }
                x509CRLEntryImpl.setCertificateIssuer(x500Principal, x500Principal);
                this.revokedMap.put(new X509CRLImpl.X509IssuerSerial(x500Principal, x509CRLEntryImpl.getSerialNumber()), x509CRLEntryImpl);
                this.revokedList.add(x509CRLEntryImpl);
                if (x509CRLEntryImpl.hasExtensions()) {
                    this.version = 1;
                }
            }
        }
    }
    
    public X509CRLImpl(X500Name x500Name, Date date, Date date2, X509CRLEntry[] array, CRLExtensions extensions) throws CRLException {
        this(x500Name, date, date2, array);
        if (extensions != null) {
            this.extensions = extensions;
            this.version = 1;
        }
    }
    
    public byte[] getEncodedInternal() throws CRLException {
        if (this.signedCRL == null) {
            throw new CRLException("Null CRL to encode");
        }
        return this.signedCRL;
    }
    
    @Override
    public byte[] getEncoded() throws CRLException {
        return (byte[])this.getEncodedInternal().clone();
    }
    
    public void encodeInfo(OutputStream outputStream) throws CRLException {
        try {
            DerOutputStream derOutputStream = new DerOutputStream();
            DerOutputStream derOutputStream2 = new DerOutputStream();
            DerOutputStream derOutputStream3 = new DerOutputStream();
            if (this.version != 0) {
                derOutputStream.putInteger(this.version);
            }
            this.infoSigAlgId.encode(derOutputStream);
            if (this.version == 0 && this.issuer.toString() == null) {
                throw new CRLException("Null Issuer DN not allowed in v1 CRL");
            }
            this.issuer.encode(derOutputStream);
            if (this.thisUpdate.getTime() < 2524608000000L) {
                derOutputStream.putUTCTime(this.thisUpdate);
            }
            else {
                derOutputStream.putGeneralizedTime(this.thisUpdate);
            }
            if (this.nextUpdate != null) {
                if (this.nextUpdate.getTime() < 2524608000000L) {
                    derOutputStream.putUTCTime(this.nextUpdate);
                }
                else {
                    derOutputStream.putGeneralizedTime(this.nextUpdate);
                }
            }
            if (!this.revokedList.isEmpty()) {
                Iterator<X509CRLEntry> iterator = this.revokedList.iterator();
                while (iterator.hasNext()) {
                    ((X509CRLEntryImpl)(X509CRLEntry)iterator.next()).encode(derOutputStream2);
                }
                derOutputStream.write((byte)48, derOutputStream2);
            }
            if (this.extensions != null) {
                this.extensions.encode(derOutputStream, true);
            }
            derOutputStream3.write((byte)48, derOutputStream);
            outputStream.write(this.tbsCertList = derOutputStream3.toByteArray());
        }
        catch (IOException ex) {
            throw new CRLException("Encoding error: " + ex.getMessage());
        }
    }
    
    @Override
    public void verify(PublicKey publicKey) throws CRLException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        this.verify(publicKey, "");
    }
    
    @Override
    public synchronized void verify(PublicKey verifiedPublicKey, String s) throws CRLException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        if (s == null) {
            s = "";
        }
        if (this.verifiedPublicKey != null && this.verifiedPublicKey.equals(verifiedPublicKey) && s.equals(this.verifiedProvider)) {
            return;
        }
        if (this.signedCRL == null) {
            throw new CRLException("Uninitialized CRL");
        }
        String name = this.sigAlgId.getName();
        Signature s2;
        if (s.length() == 0) {
            s2 = Signature.getInstance(name);
        }
        else {
            s2 = Signature.getInstance(name, s);
        }
        try {
            SignatureUtil.initVerifyWithParam(s2, verifiedPublicKey, SignatureUtil.getParamSpec(name, this.getSigAlgParams()));
        }
        catch (ProviderException ex) {
            throw new CRLException(ex.getMessage(), ex.getCause());
        }
        catch (InvalidAlgorithmParameterException cause) {
            throw new CRLException(cause);
        }
        if (this.tbsCertList == null) {
            throw new CRLException("Uninitialized CRL");
        }
        s2.update(this.tbsCertList, 0, this.tbsCertList.length);
        if (!s2.verify(this.signature)) {
            throw new SignatureException("Signature does not match.");
        }
        this.verifiedPublicKey = verifiedPublicKey;
        this.verifiedProvider = s;
        return;
    }
    
    @Override
    public synchronized void verify(PublicKey publicKey, Provider provider) throws CRLException, NoSuchAlgorithmException, InvalidKeyException, SignatureException {
        if (this.signedCRL == null) {
            throw new CRLException("Uninitialized CRL");
        }
        String name = this.sigAlgId.getName();
        Signature s;
        if (provider == null) {
            s = Signature.getInstance(name);
        }
        else {
            s = Signature.getInstance(name, provider);
        }
        try {
            SignatureUtil.initVerifyWithParam(s, publicKey, SignatureUtil.getParamSpec(name, this.getSigAlgParams()));
        }
        catch (ProviderException ex) {
            throw new CRLException(ex.getMessage(), ex.getCause());
        }
        catch (InvalidAlgorithmParameterException cause) {
            throw new CRLException(cause);
        }
        if (this.tbsCertList == null) {
            throw new CRLException("Uninitialized CRL");
        }
        s.update(this.tbsCertList, 0, this.tbsCertList.length);
        if (!s.verify(this.signature)) {
            throw new SignatureException("Signature does not match.");
        }
        this.verifiedPublicKey = publicKey;
        return;
    }
    
    public void sign(PrivateKey privateKey, String s) throws CRLException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        this.sign(privateKey, s, null);
    }
    
    public void sign(PrivateKey privateKey, String s, String provider) throws CRLException, NoSuchAlgorithmException, InvalidKeyException, NoSuchProviderException, SignatureException {
        try {
            if (this.readOnly) {
                throw new CRLException("cannot over-write existing CRL");
            }
            Signature signature;
            if (provider == null || provider.length() == 0) {
                signature = Signature.getInstance(s);
            }
            else {
                signature = Signature.getInstance(s, provider);
            }
            signature.initSign(privateKey);
            this.sigAlgId = AlgorithmId.get(signature.getAlgorithm());
            this.infoSigAlgId = this.sigAlgId;
            DerOutputStream o = new DerOutputStream();
            DerOutputStream o2 = new DerOutputStream();
            this.encodeInfo(o2);
            this.sigAlgId.encode(o2);
            signature.update(this.tbsCertList, 0, this.tbsCertList.length);
            o2.putBitString(this.signature = signature.sign());
            o.write((byte)48, o2);
            this.signedCRL = o.toByteArray();
            this.readOnly = true;
        }
        catch (IOException ex) {
            throw new CRLException("Error while encoding data: " + ex.getMessage());
        }
    }
    
    @Override
    public String toString() {
        return this.toStringWithAlgName("" + this.sigAlgId);
    }
    
    public String toStringWithAlgName(String s) {
        StringBuffer sb = new StringBuffer();
        sb.append("X.509 CRL v" + (this.version + 1) + "\n");
        if (this.sigAlgId != null) {
            sb.append("Signature Algorithm: " + s.toString() + ", OID=" + this.sigAlgId.getOID().toString() + "\n");
        }
        if (this.issuer != null) {
            sb.append("Issuer: " + this.issuer.toString() + "\n");
        }
        if (this.thisUpdate != null) {
            sb.append("\nThis Update: " + this.thisUpdate.toString() + "\n");
        }
        if (this.nextUpdate != null) {
            sb.append("Next Update: " + this.nextUpdate.toString() + "\n");
        }
        if (this.revokedList.isEmpty()) {
            sb.append("\nNO certificates have been revoked\n");
        }
        else {
            sb.append("\nRevoked Certificates: " + this.revokedList.size());
            int n = 1;
            Iterator<X509CRLEntry> iterator = this.revokedList.iterator();
            while (iterator.hasNext()) {
                sb.append("\n[" + n++ + "] " + ((X509CRLEntry)iterator.next()).toString());
            }
        }
        if (this.extensions != null) {
            Object[] array = this.extensions.getAllExtensions().toArray();
            sb.append("\nCRL Extensions: " + array.length);
            for (int i = 0; i < array.length; ++i) {
                sb.append("\n[" + (i + 1) + "]: ");
                Extension extension = (Extension)array[i];
                try {
                    if (OIDMap.getClass(extension.getExtensionId()) == null) {
                        sb.append(extension.toString());
                        byte[] extensionValue = extension.getExtensionValue();
                        if (extensionValue != null) {
                            DerOutputStream o = new DerOutputStream();
                            o.putOctetString(extensionValue);
                            sb.append("Extension unknown: DER encoded OCTET string =\n" + new HexDumpEncoder().encodeBuffer(o.toByteArray()) + "\n");
                        }
                    }
                    else {
                        sb.append(extension.toString());
                    }
                }
                catch (Exception ex) {
                    sb.append(", Error parsing this extension");
                }
            }
        }
        if (this.signature != null) {
            sb.append("\nSignature:\n" + new HexDumpEncoder().encodeBuffer(this.signature) + "\n");
        }
        else {
            sb.append("NOT signed yet\n");
        }
        return sb.toString();
    }
    
    @Override
    public boolean isRevoked(Certificate certificate) {
        return !this.revokedMap.isEmpty() && certificate instanceof X509Certificate && this.revokedMap.containsKey(new X509CRLImpl.X509IssuerSerial((X509Certificate)certificate));
    }
    
    @Override
    public int getVersion() {
        return this.version + 1;
    }
    
    @Override
    public Principal getIssuerDN() {
        return this.issuer;
    }
    
    @Override
    public X500Principal getIssuerX500Principal() {
        if (this.issuerPrincipal == null) {
            this.issuerPrincipal = this.issuer.asX500Principal();
        }
        return this.issuerPrincipal;
    }
    
    @Override
    public Date getThisUpdate() {
        return new Date(this.thisUpdate.getTime());
    }
    
    @Override
    public Date getNextUpdate() {
        if (this.nextUpdate == null) {
            return null;
        }
        return new Date(this.nextUpdate.getTime());
    }
    
    @Override
    public X509CRLEntry getRevokedCertificate(BigInteger bigInteger) {
        if (this.revokedMap.isEmpty()) {
            return null;
        }
        return (X509CRLEntry)this.revokedMap.get(new X509CRLImpl.X509IssuerSerial(this.getIssuerX500Principal(), bigInteger));
    }
    
    @Override
    public X509CRLEntry getRevokedCertificate(X509Certificate x509Certificate) {
        if (this.revokedMap.isEmpty()) {
            return null;
        }
        return (X509CRLEntry)this.revokedMap.get(new X509IssuerSerial(x509Certificate));
    }
    
    @Override
    public Set<X509CRLEntry> getRevokedCertificates() {
        if (this.revokedList.isEmpty()) {
            return null;
        }
        return new TreeSet(this.revokedList);
    }
    
    @Override
    public byte[] getTBSCertList() throws CRLException {
        if (this.tbsCertList == null) {
            throw new CRLException("Uninitialized CRL");
        }
        return (byte[])this.tbsCertList.clone();
    }
    
    @Override
    public byte[] getSignature() {
        if (this.signature == null) {
            return null;
        }
        return (byte[])this.signature.clone();
    }
    
    @Override
    public String getSigAlgName() {
        if (this.sigAlgId == null) {
            return null;
        }
        return this.sigAlgId.getName();
    }
    
    @Override
    public String getSigAlgOID() {
        if (this.sigAlgId == null) {
            return null;
        }
        return this.sigAlgId.getOID().toString();
    }
    
    @Override
    public byte[] getSigAlgParams() {
        if (this.sigAlgId == null) {
            return null;
        }
        try {
            return this.sigAlgId.getEncodedParams();
        }
        catch (IOException ex) {
            return null;
        }
    }
    
    public AlgorithmId getSigAlgId() {
        return this.sigAlgId;
    }
    
    public KeyIdentifier getAuthKeyId() throws IOException {
        AuthorityKeyIdentifierExtension authKeyIdExtension = this.getAuthKeyIdExtension();
        if (authKeyIdExtension != null) {
            return (KeyIdentifier)authKeyIdExtension.get("key_id");
        }
        return null;
    }
    
    public AuthorityKeyIdentifierExtension getAuthKeyIdExtension() throws IOException {
        return (AuthorityKeyIdentifierExtension)this.getExtension(PKIXExtensions.AuthorityKey_Id);
    }
    
    public CRLNumberExtension getCRLNumberExtension() throws IOException {
        return (CRLNumberExtension)this.getExtension(PKIXExtensions.CRLNumber_Id);
    }
    
    public BigInteger getCRLNumber() throws IOException {
        CRLNumberExtension crlNumberExtension = this.getCRLNumberExtension();
        if (crlNumberExtension != null) {
            return crlNumberExtension.get("value");
        }
        return null;
    }
    
    public DeltaCRLIndicatorExtension getDeltaCRLIndicatorExtension() throws IOException {
        return (DeltaCRLIndicatorExtension)this.getExtension(PKIXExtensions.DeltaCRLIndicator_Id);
    }
    
    public BigInteger getBaseCRLNumber() throws IOException {
        DeltaCRLIndicatorExtension deltaCRLIndicatorExtension = this.getDeltaCRLIndicatorExtension();
        if (deltaCRLIndicatorExtension != null) {
            return deltaCRLIndicatorExtension.get("value");
        }
        return null;
    }
    
    public IssuerAlternativeNameExtension getIssuerAltNameExtension() throws IOException {
        return (IssuerAlternativeNameExtension)this.getExtension(PKIXExtensions.IssuerAlternativeName_Id);
    }
    
    public IssuingDistributionPointExtension getIssuingDistributionPointExtension() throws IOException {
        return (IssuingDistributionPointExtension)this.getExtension(PKIXExtensions.IssuingDistributionPoint_Id);
    }
    
    @Override
    public boolean hasUnsupportedCriticalExtension() {
        return this.extensions != null && this.extensions.hasUnsupportedCriticalExtension();
    }
    
    @Override
    public Set<String> getCriticalExtensionOIDs() {
        if (this.extensions == null) {
            return null;
        }
        TreeSet<String> set = new TreeSet();
        for (Extension extension : this.extensions.getAllExtensions()) {
            if (extension.isCritical()) {
                set.add(extension.getExtensionId().toString());
            }
            else {
                continue;
            }
        }
        return set;
    }
    
    @Override
    public Set<String> getNonCriticalExtensionOIDs() {
        if (this.extensions == null) {
            return null;
        }
        TreeSet<String> set = new TreeSet();
        for (Extension extension : this.extensions.getAllExtensions()) {
            if (!extension.isCritical()) {
                set.add(extension.getExtensionId().toString());
            }
            else {
                continue;
            }
        }
        return set;
    }
    
    @Override
    public byte[] getExtensionValue(String s) {
        if (this.extensions == null) {
            return null;
        }
        try {
            String name = OIDMap.getName(new ObjectIdentifier(s));
            Extension value = null;
            if (name == null) {
                ObjectIdentifier obj = new ObjectIdentifier(s);
                Enumeration<Extension> elements = this.extensions.getElements();
                while (elements.hasMoreElements()) {
                    Extension extension = (Extension)elements.nextElement();
                    if (!extension.getExtensionId().equals(obj)) {
                        continue;
                    }
                    value = extension;
                    break;
                }
            }
            else {
                value = this.extensions.get(name);
            }
            if (value == null) {
                return null;
            }
            byte[] extensionValue = value.getExtensionValue();
            if (extensionValue == null) {
                return null;
            }
            DerOutputStream o = new DerOutputStream();
            o.putOctetString(extensionValue);
            return o.toByteArray();
        }
        catch (Exception ex) {
            return null;
        }
    }
    
    public Object getExtension(ObjectIdentifier objectIdentifier) {
        if (this.extensions == null) {
            return null;
        }
        return this.extensions.get(OIDMap.getName(objectIdentifier));
    }
    
    private void parse(DerValue derValue) throws CRLException, IOException {
        if (this.readOnly) {
            throw new CRLException("cannot over-write existing CRL");
        }
        if (derValue.getData() == null || derValue.tag != 48) {
            throw new CRLException("Invalid DER-encoded CRL data");
        }
        this.signedCRL = derValue.toByteArray();
        DerValue[] array = { derValue.data.getDerValue(), derValue.data.getDerValue(), derValue.data.getDerValue() };
        if (derValue.data.available() != 0) {
            throw new CRLException("signed overrun, bytes = " + derValue.data.available());
        }
        if (array[0].tag != 48) {
            throw new CRLException("signed CRL fields invalid");
        }
        this.sigAlgId = AlgorithmId.parse(array[1]);
        this.signature = array[2].getBitString();
        if (array[1].data.available() != 0) {
            throw new CRLException("AlgorithmId field overrun");
        }
        if (array[2].data.available() != 0) {
            throw new CRLException("Signature field overrun");
        }
        this.tbsCertList = array[0].toByteArray();
        DerInputStream data = array[0].data;
        this.version = 0;
        if ((byte)data.peekByte() == 2) {
            this.version = data.getInteger();
            if (this.version != 1) {
                throw new CRLException("Invalid version");
            }
        }
        AlgorithmId parse = AlgorithmId.parse(data.getDerValue());
        if (!parse.equals(this.sigAlgId)) {
            throw new CRLException("Signature algorithm mismatch");
        }
        this.infoSigAlgId = parse;
        this.issuer = new X500Name(data);
        if (this.issuer.isEmpty()) {
            throw new CRLException("Empty issuer DN not allowed in X509CRLs");
        }
        int i = (byte)data.peekByte();
        if (i == 23) {
            this.thisUpdate = data.getUTCTime();
        }
        else if (i == 24) {
            this.thisUpdate = data.getGeneralizedTime();
        }
        else {
            throw new CRLException("Invalid encoding for thisUpdate (tag=" + i + ")");
        }
        if (data.available() == 0) {
            return;
        }
        int n = (byte)data.peekByte();
        if (n == 23) {
            this.nextUpdate = data.getUTCTime();
        }
        else if (n == 24) {
            this.nextUpdate = data.getGeneralizedTime();
        }
        if (data.available() == 0) {
            return;
        }
        int n2 = (byte)data.peekByte();
        if (n2 == 48 && (n2 & 0xC0) != 0x80) {
            DerValue[] sequence = data.getSequence(4);
            X500Principal x500Principal = this.getIssuerX500Principal();
            for (int j = 0; j < sequence.length; ++j) {
                X509CRLEntryImpl o = new X509CRLEntryImpl(sequence[j]);
                x500Principal = this.getCertIssuer(o, x500Principal);
                o.setCertificateIssuer(x500Principal, x500Principal);
                this.revokedMap.put(new X509CRLImpl.X509IssuerSerial(x500Principal, o.getSerialNumber()), o);
                this.revokedList.add(o);
            }
        }
        if (data.available() == 0) {
            return;
        }
        DerValue derValue2 = data.getDerValue();
        if (derValue2.isConstructed() && derValue2.isContextSpecific((byte)0)) {
            this.extensions = new CRLExtensions(derValue2.data);
        }
        this.readOnly = true;
        return;
    }
    
    public static X500Principal getIssuerX500Principal(X509CRL x509CRL) {
        try {
            DerInputStream data = new DerInputStream(x509CRL.getEncoded()).getSequence(3)[0].data;
            if ((byte)data.peekByte() == 2) {
                data.getDerValue();
            }
            data.getDerValue();
            return new X500Principal(data.getDerValue().toByteArray());
        }
        catch (Exception cause) {
            throw new RuntimeException("Could not parse issuer", cause);
        }
    }
    
    public static byte[] getEncodedInternal(X509CRL x509CRL) throws CRLException {
        if ((x509CRL instanceof X509CRLImpl)) {
            return ((X509CRLImpl)x509CRL).getEncodedInternal();
        }
        return x509CRL.getEncoded();
    }
    
    public static X509CRLImpl toImpl(X509CRL c) throws CRLException {
        if ((c instanceof X509CRLImpl)) {
            return (X509CRLImpl)c;
        }
        return X509Factory.intern(c);
    }
    
    private X500Principal getCertIssuer(X509CRLEntryImpl x509CRLEntryImpl, X500Principal x500Principal) throws IOException {
        CertificateIssuerExtension certificateIssuerExtension = x509CRLEntryImpl.getCertificateIssuerExtension();
        if (certificateIssuerExtension != null) {
            return ((X500Name)certificateIssuerExtension.get("issuer").get(0).getName()).asX500Principal();
        }
        return x500Principal;
    }
    
    public void derEncode(OutputStream outputStream) throws IOException {
        if (this.signedCRL == null) {
            throw new IOException("Null CRL to encode");
        }
        outputStream.write((byte[])this.signedCRL.clone());
        return;
    }
    
    private static final class X509IssuerSerial implements Comparable<X509CRLImpl.X509IssuerSerial>
    {
        final X500Principal issuer;
        final BigInteger serial;
        volatile int hashcode;
        
        X509IssuerSerial(X500Principal issuer, BigInteger serial) {
            this.hashcode = 0;
            this.issuer = issuer;
            this.serial = serial;
        }
        
        X509IssuerSerial(X509Certificate x509Certificate) {
            this(x509Certificate.getIssuerX500Principal(), x509Certificate.getSerialNumber());
        }
        
        X500Principal getIssuer() {
            return this.issuer;
        }
        
        BigInteger getSerial() {
            return this.serial;
        }
        
        @Override
        public boolean equals(Object o) {
            if (o == this) {
                return true;
            }
            if (!(o instanceof X509CRLImpl.X509IssuerSerial)) {
                return false;
            }
            X509CRLImpl.X509IssuerSerial x509IssuerSerial = (X509CRLImpl.X509IssuerSerial)o;
            return this.serial.equals(x509IssuerSerial.getSerial()) && this.issuer.equals(x509IssuerSerial.getIssuer());
        }
        
        @Override
        public int hashCode() {
            if (this.hashcode == 0) {
                this.hashcode = 37 * (37 * 17 + this.issuer.hashCode()) + this.serial.hashCode();
            }
            return this.hashcode;
        }
        
        @Override
        public int compareTo(X509CRLImpl.X509IssuerSerial x509IssuerSerial) {
            int compareTo = this.issuer.toString().compareTo(x509IssuerSerial.issuer.toString());
            if (compareTo != 0) {
                return compareTo;
            }
            return this.serial.compareTo(x509IssuerSerial.serial);
        }
    }
}
